]> sigrok.org Git - libsigrok.git/blobdiff - session.c
doxygen: Updated Doxyfile to doxygen 1.8.6.
[libsigrok.git] / session.c
index 160d13fb3d6fbfe589321480ed08d6b7f7a34823..263185c186c0ac058e4e499da32600ffeb081603 100644 (file)
--- a/session.c
+++ b/session.c
 #include "libsigrok.h"
 #include "libsigrok-internal.h"
 
-/* Message logging helpers with subsystem-specific prefix string. */
-#define LOG_PREFIX "session: "
-#define sr_log(l, s, args...) sr_log(l, LOG_PREFIX s, ## args)
-#define sr_spew(s, args...) sr_spew(LOG_PREFIX s, ## args)
-#define sr_dbg(s, args...) sr_dbg(LOG_PREFIX s, ## args)
-#define sr_info(s, args...) sr_info(LOG_PREFIX s, ## args)
-#define sr_warn(s, args...) sr_warn(LOG_PREFIX s, ## args)
-#define sr_err(s, args...) sr_err(LOG_PREFIX s, ## args)
+/** @cond PRIVATE */
+#define LOG_PREFIX "session"
+/** @endcond */
 
 /**
  * @file
@@ -50,7 +45,7 @@
 
 struct source {
        int timeout;
-       sr_receive_data_callback_t cb;
+       sr_receive_data_callback cb;
        void *cb_data;
 
        /* This is used to keep track of the object (fd, pollfd or channel) which is
@@ -60,7 +55,7 @@ struct source {
 };
 
 struct datafeed_callback {
-       sr_datafeed_callback_t cb;
+       sr_datafeed_callback cb;
        void *cb_data;
 };
 
@@ -74,7 +69,8 @@ struct sr_session *session;
  * @todo Should it use the file-global "session" variable or take an argument?
  *       The same question applies to all the other session functions.
  *
- * @return A pointer to the newly allocated session, or NULL upon errors.
+ * @retval NULL Error.
+ * @retval other A pointer to the newly allocated session.
  */
 SR_API struct sr_session *sr_session_new(void)
 {
@@ -93,10 +89,10 @@ SR_API struct sr_session *sr_session_new(void)
 
 /**
  * Destroy the current session.
- *
  * This frees up all memory used by the session.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  */
 SR_API int sr_session_destroy(void)
 {
@@ -123,7 +119,8 @@ SR_API int sr_session_destroy(void)
  * The session itself (i.e., the struct sr_session) is not free'd and still
  * exists after this function returns.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  */
 SR_API int sr_session_dev_remove_all(void)
 {
@@ -145,7 +142,9 @@ SR_API int sr_session_dev_remove_all(void)
  *            be NULL. Also, sdi->driver and sdi->driver->dev_open must
  *            not be NULL.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_BUG No session exists.
  */
 SR_API int sr_session_dev_add(const struct sr_dev_inst *sdi)
 {
@@ -179,12 +178,20 @@ SR_API int sr_session_dev_add(const struct sr_dev_inst *sdi)
        session->devs = g_slist_append(session->devs, (gpointer)sdi);
 
        if (session->running) {
-               /* Adding a device to a running session. Start acquisition
-                * on that device now. */
+               /* Adding a device to a running session. Commit settings
+                * and start acquisition on that device now. */
+               if ((ret = sr_config_commit(sdi)) != SR_OK) {
+                       sr_err("Failed to commit device settings before "
+                              "starting acquisition in running session (%s)",
+                              sr_strerror(ret));
+                       return ret;
+               }
                if ((ret = sdi->driver->dev_acquisition_start(sdi,
-                                               (void *)sdi)) != SR_OK)
+                                               (void *)sdi)) != SR_OK) {
                        sr_err("Failed to start acquisition of device in "
-                                       "running session: %d", ret);
+                              "running session (%s)", sr_strerror(ret));
+                       return ret;
+               }
        }
 
        return SR_OK;
@@ -200,7 +207,8 @@ SR_API int sr_session_dev_add(const struct sr_dev_inst *sdi)
  *                The list must be freed by the caller, but not the
  *                elements pointed to.
  *
- * @return SR_OK upon success, SR_ERR upon invalid arguments.
+ * @retval SR_OK Success.
+ * @retval SR_ERR Invalid argument.
  */
 SR_API int sr_session_dev_list(GSList **devlist)
 {
@@ -218,7 +226,8 @@ SR_API int sr_session_dev_list(GSList **devlist)
 /**
  * Remove all datafeed callbacks in the current session.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  */
 SR_API int sr_session_datafeed_callback_remove_all(void)
 {
@@ -240,9 +249,10 @@ SR_API int sr_session_datafeed_callback_remove_all(void)
  *           Must not be NULL.
  * @param cb_data Opaque pointer passed in by the caller.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  */
-SR_API int sr_session_datafeed_callback_add(sr_datafeed_callback_t cb, void *cb_data)
+SR_API int sr_session_datafeed_callback_add(sr_datafeed_callback cb, void *cb_data)
 {
        struct datafeed_callback *cb_struct;
 
@@ -282,7 +292,8 @@ SR_API int sr_session_datafeed_callback_add(sr_datafeed_callback_t cb, void *cb_
  *              If FALSE, all sources have their callback run, regardless
  *              of file descriptor or timeout status.
  *
- * @return SR_OK upon success, SR_ERR on errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR Error occured.
  */
 static int sr_session_iteration(gboolean block)
 {
@@ -327,7 +338,8 @@ static int sr_session_iteration(gboolean block)
  *
  * There can only be one session at a time.
  *
- * @return SR_OK upon success, SR_ERR upon errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR Error occured.
  */
 SR_API int sr_session_start(void)
 {
@@ -352,9 +364,14 @@ SR_API int sr_session_start(void)
        ret = SR_OK;
        for (l = session->devs; l; l = l->next) {
                sdi = l->data;
+               if ((ret = sr_config_commit(sdi)) != SR_OK) {
+                       sr_err("Failed to commit device settings before "
+                              "starting acquisition (%s)", sr_strerror(ret));
+                       break;
+               }
                if ((ret = sdi->driver->dev_acquisition_start(sdi, sdi)) != SR_OK) {
                        sr_err("%s: could not start an acquisition "
-                              "(%d)", __func__, ret);
+                              "(%s)", __func__, sr_strerror(ret));
                        break;
                }
        }
@@ -367,7 +384,8 @@ SR_API int sr_session_start(void)
 /**
  * Run the session.
  *
- * @return SR_OK upon success, SR_ERR_BUG upon errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG Error occured.
  */
 SR_API int sr_session_run(void)
 {
@@ -410,7 +428,8 @@ SR_API int sr_session_run(void)
  * This must be called from within the session thread, to prevent freeing
  * resources that the session thread will try to use.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  *
  * @private
  */
@@ -449,7 +468,8 @@ SR_PRIV int sr_session_stop_sync(void)
  * to wait for the session thread to return before assuming that the session is
  * completely decommissioned.
  *
- * @return SR_OK upon success, SR_ERR_BUG if no session exists.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_BUG No session exists.
  */
 SR_API int sr_session_stop(void)
 {
@@ -487,8 +507,8 @@ static void datafeed_dump(const struct sr_datafeed_packet *packet)
                break;
        case SR_DF_LOGIC:
                logic = packet->payload;
-               sr_dbg("bus: Received SR_DF_LOGIC packet (%" PRIu64 " bytes).",
-                      logic->length);
+               sr_dbg("bus: Received SR_DF_LOGIC packet (%" PRIu64 " bytes, "
+                      "unitsize = %d).", logic->length, logic->unitsize);
                break;
        case SR_DF_ANALOG:
                analog = packet->payload;
@@ -518,7 +538,8 @@ static void datafeed_dump(const struct sr_datafeed_packet *packet)
  * @param sdi TODO.
  * @param packet The datafeed packet to send to the session bus.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
  *
  * @private
  */
@@ -552,16 +573,18 @@ SR_PRIV int sr_session_send(const struct sr_dev_inst *sdi,
  * Add an event source for a file descriptor.
  *
  * @param pollfd The GPollFD.
- * @param timeout Max time to wait before the callback is called, ignored if 0.
+ * @param[in] timeout Max time to wait before the callback is called,
+ *              ignored if 0.
  * @param cb Callback function to add. Must not be NULL.
  * @param cb_data Data for the callback function. Can be NULL.
  * @param poll_object TODO.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_MALLOC Memory allocation error.
  */
 static int _sr_session_source_add(GPollFD *pollfd, int timeout,
-       sr_receive_data_callback_t cb, void *cb_data, gintptr poll_object)
+       sr_receive_data_callback cb, void *cb_data, gintptr poll_object)
 {
        struct source *new_sources, *s;
        GPollFD *new_pollfds;
@@ -612,11 +635,12 @@ static int _sr_session_source_add(GPollFD *pollfd, int timeout,
  * @param cb Callback function to add. Must not be NULL.
  * @param cb_data Data for the callback function. Can be NULL.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_MALLOC Memory allocation error.
  */
 SR_API int sr_session_source_add(int fd, int events, int timeout,
-               sr_receive_data_callback_t cb, void *cb_data)
+               sr_receive_data_callback cb, void *cb_data)
 {
        GPollFD p;
 
@@ -634,11 +658,12 @@ SR_API int sr_session_source_add(int fd, int events, int timeout,
  * @param cb Callback function to add. Must not be NULL.
  * @param cb_data Data for the callback function. Can be NULL.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_MALLOC Memory allocation error.
  */
 SR_API int sr_session_source_add_pollfd(GPollFD *pollfd, int timeout,
-               sr_receive_data_callback_t cb, void *cb_data)
+               sr_receive_data_callback cb, void *cb_data)
 {
        return _sr_session_source_add(pollfd, timeout, cb,
                                      cb_data, (gintptr)pollfd);
@@ -653,11 +678,12 @@ SR_API int sr_session_source_add_pollfd(GPollFD *pollfd, int timeout,
  * @param cb Callback function to add. Must not be NULL.
  * @param cb_data Data for the callback function. Can be NULL.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_MALLOC Memory allocation error.
  */
 SR_API int sr_session_source_add_channel(GIOChannel *channel, int events,
-               int timeout, sr_receive_data_callback_t cb, void *cb_data)
+               int timeout, sr_receive_data_callback cb, void *cb_data)
 {
        GPollFD p;
 
@@ -676,11 +702,12 @@ SR_API int sr_session_source_add_channel(GIOChannel *channel, int events,
  *
  * @todo Add more error checks and logging.
  *
- * @param channel The channel for which the source should be removed.
+ * @param poll_object The channel for which the source should be removed.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors, SR_ERR_BUG upon
- *         internal errors.
+ * @retval SR_OK Success
+ * @retval SR_ERR_ARG Invalid arguments
+ * @retval SR_ERR_MALLOC Memory allocation error
+ * @retval SR_ERR_BUG Internal error
  */
 static int _sr_session_source_remove(gintptr poll_object)
 {
@@ -734,9 +761,10 @@ static int _sr_session_source_remove(gintptr poll_object)
  *
  * @param fd The file descriptor for which the source should be removed.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors, SR_ERR_BUG upon
- *         internal errors.
+ * @retval SR_OK Success
+ * @retval SR_ERR_ARG Invalid argument
+ * @retval SR_ERR_MALLOC Memory allocation error.
+ * @retval SR_ERR_BUG Internal error.
  */
 SR_API int sr_session_source_remove(int fd)
 {
@@ -762,9 +790,10 @@ SR_API int sr_session_source_remove_pollfd(GPollFD *pollfd)
  *
  * @param channel The channel for which the source should be removed.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR_MALLOC upon memory allocation errors, SR_ERR_BUG upon
- *         internal errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid argument.
+ * @retval SR_ERR_MALLOC Memory allocation error.
+ * @return SR_ERR_BUG Internal error.
  */
 SR_API int sr_session_source_remove_channel(GIOChannel *channel)
 {